@@ -25,7 +25,7 @@ struct selnl_msg_setenforce {
 };
 
 struct selnl_msg_policyload {
-	u_int32_t seqno;
+	uint32_t seqno;
 };
 
 #endif				/* _LINUX_SELINUX_NETLINK_H */
